import { ContentChildren, Component, ElementRef, EventEmitter, forwardRef, Input, Output, NgZone, Renderer } from '@angular/core';
import { Ion } from '../ion';
import { Config } from '../../config/config';
import { Platform } from '../../platform/platform';
import { RootNode } from '../../navigation/root-node';
const QUERY = {
    xs: '(min-width: 0px)',
    sm: '(min-width: 576px)',
    md: '(min-width: 768px)',
    lg: '(min-width: 992px)',
    xl: '(min-width: 1200px)',
    never: ''
};
/**
 * @name SplitPane
 */
export class SplitPane extends Ion {
    constructor(_zone, _plt, config, elementRef, renderer) {
        super(config, elementRef, renderer, 'split-pane');
        this._zone = _zone;
        this._plt = _plt;
        this._visible = false;
        this._init = false;
        this._mediaQuery = QUERY['md'];
        this.ionChange = new EventEmitter();
    }
    updateChildren() {
        this.mainContent = null;
        this.sideContent = null;
        this.children.forEach(child => {
            if (child !== this) {
                var isSide = child._isSideContent();
                this.setPaneCSSClass(child.getElementRef(), isSide);
                if (child.enabled) {
                    if (isSide) {
                        if (this.sideContent) {
                            console.error('split pane: side content was already set');
                        }
                        this.sideContent = child;
                    }
                    else {
                        if (this.mainContent) {
                            console.error('split pane: main content was already set');
                        }
                        this.mainContent = child;
                    }
                }
                child._setIsPane(this._visible);
            }
        });
        if (!this.mainContent) {
            console.error('split pane: one of the elements needs the "main" attribute');
        }
        if (this.mainContent === this.sideContent) {
            console.error('split pane: main and side content are the same');
        }
    }
    set when(query) {
        if (typeof query === 'boolean') {
            this._mediaQuery = query;
        }
        else {
            const defaultQuery = QUERY[query];
            this._mediaQuery = (defaultQuery)
                ? defaultQuery
                : query;
        }
        this._update();
    }
    get when() {
        return this._mediaQuery;
    }
    ngAfterViewInit() {
        this._init = true;
        this._update();
    }
    _update() {
        if (!this._init) {
            return;
        }
        // Unlisten
        this._rmListener && this._rmListener();
        this._rmListener = null;
        const query = this._mediaQuery;
        if (typeof query === 'boolean') {
            this.setVisible(query);
            return;
        }
        if (query && query.length > 0) {
            // Listen
            const callback = (query) => this.setVisible(query.matches);
            const mediaList = this._plt.win().matchMedia(query);
            mediaList.addListener(callback);
            this.setVisible(mediaList.matches);
            this._rmListener = function () {
                mediaList.removeListener(callback);
            };
        }
        else {
            this.setVisible(false);
        }
    }
    setVisible(visible) {
        if (this._visible === visible) {
            return;
        }
        this._visible = visible;
        this.setElementClass('split-pane-visible', visible);
        this.updateChildren();
        this._zone.run(() => {
            this.ionChange.emit(this);
        });
    }
    isVisible() {
        return this._visible;
    }
    setElementClass(className, add) {
        this._renderer.setElementClass(this._elementRef.nativeElement, className, add);
    }
    setPaneCSSClass(elementRef, isSide) {
        const ele = elementRef.nativeElement;
        this._renderer.setElementClass(ele, 'split-pane-side', isSide);
        this._renderer.setElementClass(ele, 'split-pane-main', !isSide);
    }
    ngOnDestroy() {
        (void 0) /* assert */;
        this._rmListener && this._rmListener();
        this._rmListener = null;
    }
    _setIsPane(isPane) {
        // Conforms to RootNode abstract class
    }
    _isSideContent() {
        return false;
    }
    get enabled() {
        return true;
    }
}
SplitPane.decorators = [
    { type: Component, args: [{
                selector: 'ion-split-pane',
                template: '<ng-content></ng-content>',
                providers: [{ provide: RootNode, useExisting: forwardRef(() => SplitPane) }]
            },] },
];
/** @nocollapse */
SplitPane.ctorParameters = [
    { type: NgZone, },
    { type: Platform, },
    { type: Config, },
    { type: ElementRef, },
    { type: Renderer, },
];
SplitPane.propDecorators = {
    'children': [{ type: ContentChildren, args: [RootNode, { descendants: false },] },],
    'when': [{ type: Input },],
    'ionChange': [{ type: Output },],
};
//# sourceMappingURL=split-pane.js.map